@@ -174,26 +174,63 @@ class TestLuaRuntime(SetupLuaRuntimeMixin, unittest.TestCase):
         self.assertEqual(4, len(table)) # as returned by Lua's "#" operator
 
     def test_iter_table(self):
-        table = self.lua.eval('{1,2,3,4,5}')
+        table = self.lua.eval('{2,3,4,5,6}')
         self.assertEqual([1,2,3,4,5], list(table))
 
     def test_iter_table_repeat(self):
-        table = self.lua.eval('{1,2,3,4,5}')
+        table = self.lua.eval('{2,3,4,5,6}')
         self.assertEqual([1,2,3,4,5], list(table)) # 1
         self.assertEqual([1,2,3,4,5], list(table)) # 2
         self.assertEqual([1,2,3,4,5], list(table)) # 3
 
+    def test_iter_array_table_values(self):
+        table = self.lua.eval('{2,3,4,5,6}')
+        self.assertEqual([2,3,4,5,6], list(table.values()))
+
+    def test_iter_array_table_repeat(self):
+        table = self.lua.eval('{2,3,4,5,6}')
+        self.assertEqual([2,3,4,5,6], list(table.values())) # 1
+        self.assertEqual([2,3,4,5,6], list(table.values())) # 2
+        self.assertEqual([2,3,4,5,6], list(table.values())) # 3
+
     def test_iter_multiple_tables(self):
         count = 10
-        tables = [ iter(self.lua.eval('{%s}' % ','.join(map(str, range(count))))) for _ in range(4) ]
+        table_values = [ self.lua.eval('{%s}' % ','.join(map(str, range(2,count+2)))).values()
+                         for _ in range(4) ]
 
         # round robin
         l = [ [] for _ in range(count) ]
         for sublist in l:
-            for table in tables:
+            for table in table_values:
                 sublist.append(_next(table))
 
-        self.assertEqual([[i]*len(tables) for i in range(1,count+1)], l)
+        self.assertEqual([[i]*len(table_values) for i in range(2,count+2)], l)
+
+    def test_iter_table_repeat(self):
+        count = 10
+        table_values = [ self.lua.eval('{%s}' % ','.join(map(str, range(2,count+2)))).values()
+                         for _ in range(4) ]
+
+        # one table after the other
+        l = [ [] for _ in range(count) ]
+        for table in table_values:
+            for sublist in l:
+                sublist.append(_next(table))
+
+        self.assertEqual([[i]*len(table_values) for i in range(2,count+2)], l)
+
+    def test_iter_table_mapping(self):
+        keys = list('abcdefg')
+        table = self.lua.eval('{%s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
+        l = list(table)
+        l.sort()
+        self.assertEqual(keys, l)
+
+    def test_iter_table_mapping_int_keys(self):
+        table = self.lua.eval('{%s}' % ','.join(['[%d]=%d' % (i,-i) for i in range(10) ]))
+        l = list(table)
+        l.sort()
+        self.assertEqual(range(10), l)
 
     def test_iter_table_keys(self):
         keys = list('abcdefg')
@@ -202,6 +239,12 @@ class TestLuaRuntime(SetupLuaRuntimeMixin, unittest.TestCase):
         l.sort()
         self.assertEqual(keys, l)
 
+    def test_iter_table_keys_int_keys(self):
+        table = self.lua.eval('{%s}' % ','.join(['[%d]=%d' % (i,-i) for i in range(10) ]))
+        l = list(table.keys())
+        l.sort()
+        self.assertEqual(range(10), l)
+
     def test_iter_table_values(self):
         keys = list('abcdefg')
         table = self.lua.eval('{%s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
@@ -209,6 +252,12 @@ class TestLuaRuntime(SetupLuaRuntimeMixin, unittest.TestCase):
         l.sort()
         self.assertEqual(list(range(len(keys))), l)
 
+    def test_iter_table_values_int_keys(self):
+        table = self.lua.eval('{%s}' % ','.join(['[%d]=%d' % (i,-i) for i in range(10) ]))
+        l = list(table.values())
+        l.sort()
+        self.assertEqual(range(-9,1), l)
+
     def test_iter_table_items(self):
         keys = list('abcdefg')
         table = self.lua.eval('{%s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
@@ -216,6 +265,23 @@ class TestLuaRuntime(SetupLuaRuntimeMixin, unittest.TestCase):
         l.sort()
         self.assertEqual(list(zip(keys,range(len(keys)))), l)
 
+    def test_iter_table_items_int_keys(self):
+        table = self.lua.eval('{%s}' % ','.join(['[%d]=%d' % (i,-i) for i in range(10) ]))
+        l = list(table.items())
+        l.sort()
+        self.assertEqual(list(zip(range(10), range(0,-10,-1))), l)
+
+    def test_iter_table_values_mixed(self):
+        keys = list('abcdefg')
+        table = self.lua.eval('{98, 99; %s}' % ','.join(['%s=%d' % (c,i) for i,c in enumerate(keys)]))
+        l = list(table.values())
+        l.sort()
+        self.assertEqual(list(range(len(keys))) + [98, 99], l)
+
+    def test_error_iter_number(self):
+        func = self.lua.eval('1')
+        self.assertRaises(TypeError, list, func)
+
     def test_error_iter_function(self):
         func = self.lua.eval('function() return 1 end')
         self.assertRaises(TypeError, list, func)
